import unittest

from fn import Stream, iters


class StreamTestCase(unittest.TestCase):

    def test_from_list(self):
        s = Stream() << [1, 2, 3, 4, 5]
        self.assertEqual([1, 2, 3, 4, 5], list(s))
        self.assertEqual(2, s[1])
        self.assertEqual([1, 2], list(s[0:2]))

    def test_from_iterator(self):
        s = Stream() << range(6) << [6, 7]
        self.assertEqual([0, 1, 2, 3, 4, 5, 6, 7], list(s))

    def test_from_generator(self):
        def gen():
            yield 1
            yield 2
            yield 3

        s = Stream() << gen << (4, 5)
        assert list(s) == [1, 2, 3, 4, 5]

    def test_lazy_slicing(self):
        s = Stream() << iters.range(10)
        self.assertEqual(s.cursor(), 0)

        s_slice = s[:5]
        self.assertEqual(s.cursor(), 0)
        self.assertEqual(len(list(s_slice)), 5)

    def test_lazy_slicing_recursive(self):
        s = Stream() << iters.range(10)
        sf = s[1:3][0:2]

        self.assertEqual(s.cursor(), 0)
        self.assertEqual(len(list(sf)), 2)

    def test_fib_infinite_stream(self):
        from operator import add

        f = Stream()
        fib = f << [0, 1] << iters.map(add, f, iters.drop(1, f))

        self.assertEqual(
            [0, 1, 1, 2, 3, 5, 8, 13, 21, 34],
            list(iters.take(10, fib))
        )
        self.assertEqual(6765, fib[20])
        self.assertEqual(
            [832040, 1346269, 2178309, 3524578, 5702887],
            list(fib[30:35])
        )
        # 35 elements should be already evaluated
        self.assertEqual(fib.cursor(), 35)

    def test_origin_param(self):
        self.assertEqual([100], list(Stream(100)))
        self.assertEqual([1, 2, 3], list(Stream(1, 2, 3)))
        self.assertEqual(
            [1, 2, 3, 10, 20, 30],
            list(Stream(1, 2, 3) << [10, 20, 30])
        )

    def test_origin_param_string(self):
        self.assertEqual(["stream"], list(Stream("stream")))



import unittest

from fn.immutable import PairingHeap, SkewHeap


class UnionBasedHeapsTestCase(unittest.TestCase):

    def _heap_basic_operations(self, cls):
        # Create new heap with 3 elements
        s1 = cls(10)
        s2 = s1.insert(30)
        s3 = s2.insert(20)

        # Extract elements one-by-one
        el1, sx1 = s3.extract()
        el2, sx2 = sx1.extract()
        el3, sx3 = sx2.extract()

        # Check elements ordering
        self.assertEqual(10, el1)
        self.assertEqual(20, el2)
        self.assertEqual(30, el3)

        # Check that previous heap are persistent
        el22, _ = sx1.extract()
        self.assertEqual(20, el22)

    def _heap_iterator(self, cls):
        # Create new heap with 5 elements
        h = cls(10)
        h = h.insert(30)
        h = h.insert(20)
        h = h.insert(5)
        h = h.insert(100)

        # Convert to list using iterator
        self.assertEqual([5, 10, 20, 30, 100], list(h))

    def _heap_custom_compare(self, cls):
        h = cls(cmp=lambda a, b: len(a) - len(b))
        h = h.insert("give")
        h = h.insert("few words")
        h = h.insert("about")
        h = h.insert("union heaps")
        h = h.insert("implementation")

        # Convert to list using iterator
        self.assertEqual(["give",
                          "about",
                          "few words",
                          "union heaps",
                          "implementation"], list(h))

    def _heap_compare_with_keyfunc(self, cls):
        from operator import itemgetter

        # Create new heap with 5 elements
        h = cls(key=itemgetter(1))
        h = h.insert((10, 10))
        h = h.insert((30, 15))
        h = h.insert((20, 110))
        h = h.insert((40, -10))
        h = h.insert((50, 100))

        # Convert to list using iterator
        self.assertEqual(
            [(40, -10), (10, 10), (30, 15), (50, 100), (20, 110)],
            list(h)
        )

    def test_skew_heap_basic(self):
        self._heap_basic_operations(SkewHeap)

    def test_pairing_heap_basic(self):
        self._heap_basic_operations(PairingHeap)

    def test_skew_heap_iterator(self):
        self._heap_iterator(SkewHeap)

    def test_pairing_heap_iterator(self):
        self._heap_iterator(PairingHeap)

    def test_skew_heap_key_func(self):
        self._heap_compare_with_keyfunc(SkewHeap)

    def test_pairing_heap_key_func(self):
        self._heap_compare_with_keyfunc(PairingHeap)

    def test_skew_heap_cmp_func(self):
        self._heap_custom_compare(SkewHeap)

    def test_pairing_heap_cmp_func(self):
        self._heap_custom_compare(PairingHeap)



import operator
import sys
import unittest

from fn import monad


class InstanceChecker(object):
    if sys.version_info[0] == 2 and sys.version_info[1] <= 6:
        def assertIsInstance(self, inst, cls):
            self.assertTrue(isinstance(inst, cls))


class OptionTestCase(unittest.TestCase, InstanceChecker):

    def test_create_option(self):
        self.assertIsInstance(monad.Option("A"), monad.Full)
        self.assertIsInstance(monad.Option(10), monad.Full)
        self.assertIsInstance(monad.Option(10, lambda x: x > 7), monad.Full)
        self.assertIsInstance(monad.Option(None), monad.Empty)
        self.assertIsInstance(monad.Option(False), monad.Full)
        self.assertIsInstance(monad.Option(0), monad.Full)
        self.assertIsInstance(monad.Option(False, checker=bool), monad.Empty)
        self.assertIsInstance(monad.Option(0, checker=bool), monad.Empty)
        self.assertIsInstance(monad.Option(10, lambda x: x > 70), monad.Empty)

    def test_map_filter(self):
        class Request(dict):
            def parameter(self, name):
                return monad.Option(self.get(name, None))

        r = Request(testing="Fixed", empty="   ")

        # full chain
        self.assertEqual("FIXED", r.parameter("testing")
                                   .map(operator.methodcaller("strip"))
                                   .filter(len)
                                   .map(operator.methodcaller("upper"))
                                   .get_or(""))

        # breaks on filter
        self.assertEqual("", r.parameter("empty")
                              .map(operator.methodcaller("strip"))
                              .filter(len)
                              .map(operator.methodcaller("upper"))
                              .get_or(""))

        # breaks on parameter
        self.assertEqual("", r.parameter("missed")
                              .map(operator.methodcaller("strip"))
                              .filter(len)
                              .map(operator.methodcaller("upper"))
                              .get_or(""))

    def test_empty_check(self):
        self.assertTrue(monad.Empty().empty)
        self.assertTrue(monad.Option(None).empty)
        self.assertTrue(monad.Option.from_call(lambda: None).empty)
        self.assertFalse(monad.Option(10).empty)
        self.assertFalse(monad.Full(10).empty)

    def test_lazy_orcall(self):
        def from_mimetype(request):
            # you can return both value or Option
            return request.get("mimetype", None)

        def from_extension(request):
            # you can return both value or Option
            return monad.Option(request.get("url", None))\
                        .map(lambda s: s.split(".")[-1])

        # extract value from extension
        r = dict(url="myfile.png")
        self.assertEqual(
            "PNG",
            monad.Option(
                r.get("type", None)
            ).or_call(
                from_mimetype, r
            ).or_call(
                from_extension, r
            ).map(
                operator.methodcaller("upper")
            ).get_or("")
        )

        # extract value from mimetype
        r = dict(url="myfile.svg", mimetype="png")
        self.assertEqual(
            "PNG",
            monad.Option(
                r.get("type", None)
            ).or_call(
                from_mimetype, r
            ).or_call(
                from_extension, r
            ).map(
                operator.methodcaller("upper")
            ).get_or("")
        )

        # type is set directly
        r = dict(url="myfile.jpeg", mimetype="svg", type="png")
        self.assertEqual(
            "PNG",
            monad.Option(
                r.get("type", None)
            ).or_call(
                from_mimetype, r
            ).or_call(
                from_extension, r
            ).map(
                operator.methodcaller("upper")
            ).get_or("")
        )

    def test_optionable_decorator(self):
        class Request(dict):
            @monad.optionable
            def parameter(self, name):
                return self.get(name, None)

        r = Request(testing="Fixed", empty="   ")

        # full chain
        self.assertEqual("FIXED", r.parameter("testing")
                                   .map(operator.methodcaller("strip"))
                                   .filter(len)
                                   .map(operator.methodcaller("upper"))
                                   .get_or(""))

    def test_stringify(self):
        self.assertEqual("Full(10)", str(monad.Full(10)))
        self.assertEqual("Full(in box!)", str(monad.Full("in box!")))
        self.assertEqual("Empty()", str(monad.Empty()))
        self.assertEqual("Empty()", str(monad.Option(None)))

    def test_option_repr(self):
        self.assertEqual("Full(10)", repr(monad.Full(10)))
        self.assertEqual("Full(in box!)", repr(monad.Full("in box!")))
        self.assertEqual("Empty()", repr(monad.Empty()))
        self.assertEqual("Empty()", repr(monad.Option(None)))

    def test_static_constructor(self):
        self.assertEqual(monad.Empty(),  monad.Option.from_value(None))
        self.assertEqual(monad.Full(10), monad.Option.from_value(10))
        self.assertEqual(monad.Empty(),  monad.Option.from_call(lambda: None))
        self.assertEqual(
            monad.Full(10),
            monad.Option.from_call(operator.add, 8, 2)
        )
        self.assertEqual(
            monad.Empty(),
            monad.Option.from_call(lambda d, k: d[k],
                                   {"a": 1}, "b", exc=KeyError)
        )

    def test_flatten_operation(self):
        self.assertEqual(monad.Empty(), monad.Empty(monad.Empty()))
        self.assertEqual(monad.Empty(), monad.Empty(monad.Full(10)))
        self.assertEqual(monad.Empty(), monad.Full(monad.Empty()))
        self.assertEqual("Full(20)", str(monad.Full(monad.Full(20))))



import unittest

from fn import _, underscore
from fn.uniform import reduce


class UnderscoreTestCase(unittest.TestCase):

    def test_identity_default(self):
        self.assertEqual(10, _(10))

    def test_arithmetic(self):
        # operator +
        self.assertEqual(7, (_ + 2)(5))
        self.assertEqual([10, 11, 12], list(map(_ + 10, [0, 1, 2])))
        # operator -
        self.assertEqual(3, (_ - 2)(5))
        self.assertEqual(13, (_ - 2 + 10)(5))
        self.assertEqual([0, 1, 2], list(map(_ - 10, [10, 11, 12])))
        # operator *
        self.assertEqual(10, (_ * 2)(5))
        self.assertEqual(50, (_ * 2 + 40)(5))
        self.assertEqual([0, 10, 20], list(map(_ * 10, [0, 1, 2])))
        # operator /
        self.assertEqual(5, (_ / 2)(10))
        self.assertEqual(6, (_ / 2 + 1)(10))
        self.assertEqual([1, 2, 3], list(map(_ / 10, [10, 20, 30])))
        # operator **
        self.assertEqual(100, (_ ** 2)(10))
        # operator %
        self.assertEqual(1, (_ % 2)(11))
        # operator <<
        self.assertEqual(32, (_ << 2)(8))
        # operator >>
        self.assertEqual(2, (_ >> 2)(8))
        # operator (-a)
        self.assertEqual(10,  (-_)(-10))
        self.assertEqual(-10, (-_)(10))
        # operator (+a)
        self.assertEqual(10,  (+_)(10))
        self.assertEqual(-10, (+_)(-10))
        # operator (~a)
        self.assertEqual(-11, (~_)(10))

    def test_arithmetic_multiple(self):
        self.assertEqual(10, (_ + _)(5, 5))
        self.assertEqual(0, (_ - _)(5, 5))
        self.assertEqual(25, (_ * _)(5, 5))
        self.assertEqual(1, (_ / _)(5, 5))

    def test_arithmetic_swap(self):
        # operator +
        self.assertEqual(7, (2 + _)(5))
        self.assertEqual([10, 11, 12], list(map(10 + _, [0, 1, 2])))
        # operator -
        self.assertEqual(3, (8 - _)(5))
        self.assertEqual(13, (8 - _ + 10)(5))
        self.assertEqual([10, 9, 8], list(map(10 - _, [0, 1, 2])))
        # operator *
        self.assertEqual(10, (2 * _)(5))
        self.assertEqual(50, (2 * _ + 40)(5))
        self.assertEqual([0, 10, 20], list(map(10 * _, [0, 1, 2])))
        # operator /
        self.assertEqual(5, (10 / _)(2))
        self.assertEqual(6, (10 / _ + 1)(2))
        self.assertEqual([10, 5, 2], list(map(100 / _, [10, 20, 50])))
        # operator **
        self.assertEqual(100, (10**_)(2))
        # operator %
        self.assertEqual(1, (11 % _)(2))
        # operator <<
        self.assertEqual(32, (8 << _)(2))
        # operator >>
        self.assertEqual(2, (8 >> _)(2))

    def test_bitwise(self):
        # and
        self.assertTrue((_ & 1)(1))
        self.assertFalse((_ & 1)(0))
        self.assertFalse((_ & 0)(1))
        self.assertFalse((_ & 0)(0))
        # or
        self.assertTrue((_ | 1)(1))
        self.assertTrue((_ | 1)(0))
        self.assertTrue((_ | 0)(1))
        self.assertFalse((_ | 0)(0))
        # xor
        self.assertTrue((_ ^ 1)(0))
        self.assertTrue((_ ^ 0)(1))
        self.assertFalse((_ ^ 1)(1))
        self.assertFalse((_ ^ 0)(0))

    def test_bitwise_swap(self):
        # and
        self.assertTrue((1 & _)(1))
        self.assertFalse((1 & _)(0))
        self.assertFalse((0 & _)(1))
        self.assertFalse((0 & _)(0))
        # or
        self.assertTrue((1 | _)(1))
        self.assertTrue((1 | _)(0))
        self.assertTrue((0 | _)(1))
        self.assertFalse((0 | _)(0))
        # xor
        self.assertTrue((1 ^ _)(0))
        self.assertTrue((0 ^ _)(1))
        self.assertFalse((1 ^ _)(1))
        self.assertFalse((0 ^ _)(0))

    def test_getattr(self):
        class GetattrTest(object):
            def __init__(self):
                self.doc = "TestCase"

        self.assertEqual("TestCase", (_.doc)(GetattrTest()))
        self.assertEqual("TestCaseTestCase", (_.doc * 2)(GetattrTest()))
        self.assertEqual(
            "TestCaseTestCase", (_.doc + _.doc)(GetattrTest(), GetattrTest())
        )

    def test_call_method(self):
        self.assertEqual(["test", "case"], (_.call("split"))("test case"))
        self.assertEqual("str", _.__name__(str))

    def test_call_method_args(self):
        self.assertEqual(["test", "case"], (_.call("split", "-"))("test-case"))
        self.assertEqual(["test-case"], (_.call("split", "-", 0))("test-case"))

    def test_call_method_kwargs(self):
        test_dict = {'num': 23}
        _.call("update", num=42)(test_dict)
        self.assertEqual({'num': 42}, (test_dict))

    def test_comparator(self):
        self.assertTrue((_ < 7)(1))
        self.assertFalse((_ < 7)(10))
        self.assertTrue((_ > 20)(25))
        self.assertFalse((_ > 20)(0))
        self.assertTrue((_ <= 7)(6))
        self.assertTrue((_ <= 7)(7))
        self.assertFalse((_ <= 7)(8))
        self.assertTrue((_ >= 7)(8))
        self.assertTrue((_ >= 7)(7))
        self.assertFalse((_ >= 7)(6))
        self.assertTrue((_ == 10)(10))
        self.assertFalse((_ == 10)(9))

    def test_none(self):
        self.assertTrue((_ == None)(None))  # noqa: E711

        class pushlist(list):
            def __lshift__(self, item):
                self.append(item)
                return self

        self.assertEqual([None], (_ << None)(pushlist()))

    def test_comparator_multiple(self):
        self.assertTrue((_ < _)(1, 2))
        self.assertFalse((_ < _)(2, 1))
        self.assertTrue((_ > _)(25, 20))
        self.assertFalse((_ > _)(20, 25))
        self.assertTrue((_ <= _)(6, 7))
        self.assertTrue((_ <= _)(7, 7))
        self.assertFalse((_ <= _)(8, 7))
        self.assertTrue((_ >= _)(8, 7))
        self.assertTrue((_ >= _)(7, 7))
        self.assertFalse((_ >= _)(6, 7))
        self.assertTrue((_ == _)(10, 10))
        self.assertFalse((_ == _)(9, 10))

    def test_comparator_filter(self):
        self.assertEqual([0, 1, 2], list(filter(_ < 5, [0, 1, 2, 10, 11, 12])))

    def test_slicing(self):
        self.assertEqual(0,       (_[0])(list(range(10))))
        self.assertEqual(9,       (_[-1])(list(range(10))))
        self.assertEqual([3, 4, 5], (_[3:])(list(range(6))))
        self.assertEqual([0, 1, 2], (_[:3])(list(range(10))))
        self.assertEqual([1, 2, 3], (_[1:4])(list(range(10))))
        self.assertEqual([0, 2, 4], (_[0:6:2])(list(range(10))))

    def test_slicing_multiple(self):
        self.assertEqual(0, (_[_])(range(10), 0))
        self.assertEqual(8, (_[_ * (-1)])(range(10), 2))

    def test_arity_error(self):
        self.assertRaises(underscore.ArityError, _, 1, 2)
        self.assertRaises(underscore.ArityError, _ + _, 1)
        # can be catched as TypeError
        self.assertRaises(TypeError, _, 1, 2)
        self.assertRaises(TypeError, _ + _, 1)

    def test_more_than_2_operations(self):
        self.assertEqual(12, (_ * 2 + 10)(1))
        self.assertEqual(6,  (_ + _ + _)(1, 2, 3))
        self.assertEqual(10, (_ + _ + _ + _)(1, 2, 3, 4))
        self.assertEqual(7,  (_ + _ * _)(1, 2, 3))

    def test_string_converting(self):
        self.assertEqual("(x1) => x1", str(_))

        self.assertEqual("(x1) => (x1 + 2)",  str(_ + 2))
        self.assertEqual("(x1) => (x1 - 2)",  str(_ - 2))
        self.assertEqual("(x1) => (x1 * 2)",  str(_ * 2))
        self.assertEqual("(x1) => (x1 / 2)",  str(_ / 2))
        self.assertEqual("(x1) => (x1 % 2)",  str(_ % 2))
        self.assertEqual("(x1) => (x1 ** 2)", str(_ ** 2))

        self.assertEqual("(x1) => (x1 & 2)", str(_ & 2))
        self.assertEqual("(x1) => (x1 | 2)", str(_ | 2))
        self.assertEqual("(x1) => (x1 ^ 2)", str(_ ^ 2))

        self.assertEqual("(x1) => (x1 >> 2)", str(_ >> 2))
        self.assertEqual("(x1) => (x1 << 2)", str(_ << 2))

        self.assertEqual("(x1) => (x1 < 2)",  str(_ < 2))
        self.assertEqual("(x1) => (x1 > 2)",  str(_ > 2))
        self.assertEqual("(x1) => (x1 <= 2)", str(_ <= 2))
        self.assertEqual("(x1) => (x1 >= 2)", str(_ >= 2))
        self.assertEqual("(x1) => (x1 == 2)", str(_ == 2))
        self.assertEqual("(x1) => (x1 != 2)", str(_ != 2))

        self.assertEqual("(x1) => ((x1 * 2) + 1)", str((_ * 2 + 1)))

    def test_rigthside_string_converting(self):
        self.assertEqual("(x1) => (2 + x1)",  str(2 + _))
        self.assertEqual("(x1) => (2 - x1)",  str(2 - _))
        self.assertEqual("(x1) => (2 * x1)",  str(2 * _))
        self.assertEqual("(x1) => (2 / x1)",  str(2 / _))
        self.assertEqual("(x1) => (2 % x1)",  str(2 % _))
        self.assertEqual("(x1) => (2 ** x1)", str(2 ** _))

        self.assertEqual("(x1) => (2 & x1)", str(2 & _))
        self.assertEqual("(x1) => (2 | x1)", str(2 | _))
        self.assertEqual("(x1) => (2 ^ x1)", str(2 ^ _))

        self.assertEqual("(x1) => (2 >> x1)", str(2 >> _))
        self.assertEqual("(x1) => (2 << x1)", str(2 << _))

    def test_unary_string_converting(self):
        self.assertEqual("(x1) => (+x1)", str(+_))
        self.assertEqual("(x1) => (-x1)", str(-_))
        self.assertEqual("(x1) => (~x1)", str(~_))

    def test_multiple_string_converting(self):
        self.assertEqual("(x1, x2) => (x1 + x2)", str(_ + _))
        self.assertEqual("(x1, x2) => (x1 * x2)", str(_ * _))
        self.assertEqual("(x1, x2) => (x1 - x2)", str(_ - _))
        self.assertEqual("(x1, x2) => (x1 / x2)", str(_ / _))
        self.assertEqual("(x1, x2) => (x1 % x2)", str(_ % _))
        self.assertEqual("(x1, x2) => (x1 ** x2)", str(_ ** _))

        self.assertEqual("(x1, x2) => (x1 & x2)", str(_ & _))
        self.assertEqual("(x1, x2) => (x1 | x2)", str(_ | _))
        self.assertEqual("(x1, x2) => (x1 ^ x2)", str(_ ^ _))

        self.assertEqual("(x1, x2) => (x1 >> x2)", str(_ >> _))
        self.assertEqual("(x1, x2) => (x1 << x2)", str(_ << _))

        self.assertEqual("(x1, x2) => (x1 > x2)",  str(_ > _))
        self.assertEqual("(x1, x2) => (x1 < x2)",  str(_ < _))
        self.assertEqual("(x1, x2) => (x1 >= x2)", str(_ >= _))
        self.assertEqual("(x1, x2) => (x1 <= x2)", str(_ <= _))
        self.assertEqual("(x1, x2) => (x1 == x2)", str(_ == _))
        self.assertEqual("(x1, x2) => (x1 != x2)", str(_ != _))

        self.assertEqual(
            "(x1, x2) => (((x1 / x2) - 1) * 100)",
            str((_ / _ - 1) * 100)
        )

    def test_reverse_string_converting(self):
        self.assertEqual("(x1, x2, x3) => ((x1 + x2) + x3)", str(_ + _ + _))
        self.assertEqual("(x1, x2, x3) => (x1 + (x2 * x3))", str(_ + _ * _))

        self.assertEqual("(x1) => (1 + (2 * x1))", str((1 + 2 * _)))

    def test_multi_underscore_string_converting(self):
        self.assertEqual("(x1) => (x1 + '_')", str(_ + "_"))
        self.assertEqual(
            "(x1, x2) => getattr((x1 + x2), '__and_now__')",
            str((_ + _).__and_now__)
        )
        self.assertEqual(
            "(x1, x2) => x1['__name__'][x2]",
            str(_['__name__'][_])
        )

    def test_repr(self):
        self.assertEqual(_ / 2, eval(repr(_ / 2)))
        self.assertEqual(_ + _, eval(repr(_ + _)))
        self.assertEqual(_ + _ * _, eval(repr(_ + _ * _)))

    def test_repr_parse_str(self):
        self.assertEqual('=> ' + _, eval(repr('=> ' + _)))
        self.assertEqual(
            reduce(lambda f, n: f.format(n), ('({0} & _)',) * 11).format('_'),
            repr(reduce(_ & _, (_,) * 12)),
        )



from .finger import Deque
from .heap import PairingHeap, SkewHeap
from .list import Deque as ListDeque
from .list import LinkedList, Queue, Stack
from .trie import Vector

# from .dict import Dict
# from .tree import FingerTree


__all__ = [
    "Deque",
    "PairingHeap",
    "SkewHeap",
    "LinkedList",
    "LinkedList",
    "Queue",
    "Stack",
    "Vector",
    "ListDeque",
]



import unittest

from fn.immutable import Deque


class FingerTreeDequeTestCase(unittest.TestCase):

    def test_deque_basic_operations(self):
        d1 = Deque()
        d2 = d1.push_back(1)
        d3 = d2.push_back(2)
        d4 = d3.push_back(3)
        d5 = d4.push_front(10)
        d6 = d5.push_front(20)
        self.assertEqual(1, d4.head())
        self.assertEqual(3, d4.last())
        self.assertEqual(20, d6.head())
        self.assertEqual(3, d6.last())

    # FIXME: write this test.
    def test_deque_num_of_elements(self):
        pass

    def test_deque_is_empty(self):
        self.assertTrue(Deque().is_empty())
        self.assertFalse(Deque().push_back(1).is_empty())
        self.assertTrue(Deque().push_back(1).tail().is_empty())

    def test_iterator(self):
        self.assertEqual([], list(Deque()))
        self.assertEqual(
            [1, 2, 3],
            list(Deque().push_back(1).push_back(2).push_back(3))
        )
        self.assertEqual(
            60,
            sum(Deque().push_back(10).push_front(20).push_back(30))
        )
        self.assertEqual(
            sum(range(1, 20)),
            sum(Deque.from_iterable(range(1, 20)))
        )


if __name__ == '__main__':
    unittest.main()



import unittest

from fn.immutable import Vector
from fn.uniform import reduce


class VectorTestCase(unittest.TestCase):

    def test_cons_operation(self):
        v = Vector()
        self.assertEqual(0, len(v))
        v1 = v.cons(10)
        self.assertEqual(1, len(v1))
        self.assertEqual(0, len(v))  # previous value didn't change
        up = reduce(lambda acc, el: acc.cons(el), range(513), Vector())
        self.assertEqual(513, len(up))

    def test_assoc_get_operations(self):
        v = Vector()
        v1 = v.assoc(0, 10)
        v2 = v1.assoc(1, 20)
        v3 = v2.assoc(2, 30)
        self.assertEqual(10, v3.get(0))
        self.assertEqual(20, v3.get(1))
        self.assertEqual(30, v3.get(2))
        # check persistence
        v4 = v2.assoc(2, 50)
        self.assertEqual(30, v3.get(2))
        self.assertEqual(50, v4.get(2))
        # long vector
        up = reduce(lambda acc, el: acc.assoc(el, el*2), range(1500), Vector())
        self.assertEqual(2800, up.get(1400))
        self.assertEqual(2998, up.get(1499))

    def test_pop_operations(self):
        v = reduce(lambda acc, el: acc.cons(el), range(2000), Vector())
        self.assertEqual(1999, len(v.pop()))
        self.assertEqual(list(range(1999)), list(v.pop()))

    def test_vector_iterator(self):
        v = reduce(lambda acc, el: acc.assoc(el, el+1), range(1500), Vector())
        self.assertEqual(list(range(1, 1501)), list(v))
        self.assertEqual(1125750, sum(v))

    def test_index_error(self):
        v = reduce(lambda acc, el: acc.assoc(el, el+2), range(50), Vector())
        self.assertRaises(IndexError, v.get, -1)
        self.assertRaises(IndexError, v.get, 50)
        self.assertRaises(IndexError, v.get, 52)

    def test_setitem_should_not_be_implemented(self):
        def f():
            v = Vector().cons(20)
            v[0] = 10
        self.assertRaises(NotImplementedError, f)

    # FIXME: Write this test.
    def test_subvector_operation(self):
        pass



import unittest

from fn.immutable import Queue


class BankerQueueTestCase(unittest.TestCase):

    def test_queue_basic_operations(self):
        q1 = Queue()
        q2 = q1.enqueue(1)
        q3 = q2.enqueue(10)
        q4 = q3.enqueue(100)
        self.assertEqual(1, q4.dequeue()[0])
        self.assertEqual(1, q3.dequeue()[0])
        self.assertEqual(1, q2.dequeue()[0])
        v1, q5 = q4.dequeue()
        v2, q6 = q5.dequeue()
        v3, q7 = q6.dequeue()
        self.assertEqual(1, v1)
        self.assertEqual(10, v2)
        self.assertEqual(100, v3)
        self.assertEqual(0, len(q7))

    def test_queue_num_of_elements(self):
        self.assertEqual(0, len(Queue()))
        self.assertEqual(3, len(Queue().enqueue(1).enqueue(2).enqueue(3)))

    def test_queue_is_empty(self):
        self.assertTrue(Queue().is_empty())
        self.assertFalse(Queue().enqueue(1).is_empty())
        self.assertTrue(Queue().enqueue(1).dequeue()[1].is_empty())

    def test_dequeue_from_empty(self):
        self.assertRaises(ValueError, Queue().dequeue)

    def test_iterator(self):
        self.assertEqual([], list(Queue()))
        self.assertEqual(
            [1, 2, 3],
            list(Queue().enqueue(1).enqueue(2).enqueue(3))
        )
        self.assertEqual(60, sum(Queue().enqueue(10).enqueue(20).enqueue(30)))



import operator
import sys
import unittest

from fn import recur


class TrampolineTestCase(unittest.TestCase):

    def test_tco_decorator(self):

        def recur_accumulate(origin, f=operator.add, acc=0):
            n = next(origin, None)
            if n is None:
                return acc
            return recur_accumulate(origin, f, f(acc, n))

        # this works normally
        self.assertEqual(10, recur_accumulate(iter(range(5))))

        limit = sys.getrecursionlimit() * 10
        # such count of recursive calls should fail on CPython,
        # for PyPy we skip this test cause on PyPy the limit is
        # approximative and checked at a lower level
        if not hasattr(sys, 'pypy_version_info'):
            self.assertRaises(
                RuntimeError, recur_accumulate, iter(range(limit))
            )

        # with recur decorator it should run without problems
        @recur.tco
        def tco_accumulate(origin, f=operator.add, acc=0):
            n = next(origin, None)
            if n is None:
                return False, acc
            return True, (origin, f, f(acc, n))

        self.assertEqual(sum(range(limit)), tco_accumulate(iter(range(limit))))

    def test_tco_different_functions(self):

        @recur.tco
        def recur_inc2(curr, acc=0):
            if curr == 0:
                return False, acc
            return recur_dec, (curr-1, acc+2)

        @recur.tco
        def recur_dec(curr, acc=0):
            if curr == 0:
                return False, acc
            return recur_inc2, (curr-1, acc-1)

        self.assertEqual(5000, recur_inc2(10000))



import unittest

from fn.func import curried, _has_type_hint_support


class Curriedtest(unittest.TestCase):

    def _assert_instance(self, expected, acutal):
        self.assertEqual(expected.__module__, acutal.__module__)
        self.assertEqual(expected.__name__, acutal.__name__)

        if _has_type_hint_support:
            self.assertEqual(expected.__annotations__, acutal.__annotations__)

    def test_curried_wrapper(self):

        @curried
        def _child(a, b, c, d):
            return a + b + c + d

        @curried
        def _moma(a, b):
            return _child(a, b)

        res1 = _moma(1)
        self._assert_instance(_moma, res1)
        res2 = res1(2)
        self._assert_instance(_child, res2)
        res3 = res2(3)
        self._assert_instance(_child, res3)
        res4 = res3(4)

        self.assertEqual(res4, 10)

    @unittest.skipIf(not _has_type_hint_support, "Type hint aren't supported")
    def test_curried_with_annotations_when_they_are_supported(self):

        def _custom_sum(a, b, c, d):
            return a + b + c + d

        _custom_sum.__annotations__ = {
                                        'a': int,
                                        'b': int,
                                        'c': int,
                                        'd': int,
                                        'return': int
                                      }

        custom_sum = curried(_custom_sum)

        res1 = custom_sum(1)
        self._assert_instance(custom_sum, res1)
        res2 = res1(2)
        self._assert_instance(custom_sum, res2)
        res3 = res2(3)
        self._assert_instance(custom_sum, res3)
        res4 = res3(4)

        self.assertEqual(res4, 10)



from fn.op import foldr
from fn.uniform import reduce


class LinkedList(object):
    """Represents simplest singly linked list. Doesn't distinguish
    between empty and not-empty list (taking head of empty list will
    return None as a result).

    More about Linked List data structure on Wikipedia:
    [1] http://en.wikipedia.org/wiki/Linked_list

    Usage:

    >>> from fn.immutable import LinkedList
    >>> l = LinkedList()
    >>> l.cons(10)
    <fn.immutable.list.LinkedList object at 0x10acfbb00>
    >>> l.cons(10).cons(20).cons(30).head
    30
    >>> l.cons(10).cons(20).cons(30).tail
    <fn.immutable.list.LinkedList object at 0x10acfbc20>
    >>> l.cons(10).cons(20).cons(30).tail.head
    20
    >>> len(l.cons(10).cons(20).cons(30))
    3
    >>> list(l.cons(10).cons(20).cons(30))
    [30, 20, 10]
    >>> list(l + 100 + 110 + 120)
    [120, 110, 100]
    """

    __slots__ = ("head", "tail", "_count")

    def __init__(self, head=None, tail=None):
        self.head = head
        self.tail = tail
        self._count = 0 if tail is None else (len(tail) + 1)

    def cons(self, el):
        return self.__class__(el, self)

    def __add__(self, el):
        return self.cons(el)

    def __radd__(self, el):
        return self.cons(el)

    def __iter__(self):
        while self:
            yield self.head
            self = self.tail

    def __len__(self):
        return self._count

    def __nonzero__(self):
        return self.tail is not None

    def __bool__(self):
        return len(self) > 0

    @staticmethod
    def from_iterable(it):
        ''' iterable -> LinkedList

        produces LinkedList with the contents of the consumed iterable
        '''
        return foldr(lambda x, y: y.cons(x), LinkedList())(tuple(it))


class Stack(LinkedList):
    """Technically it's a LinkedList, but it provides more familiar
    API for Stack operations: push and pop. It also distinguishes between
    empty and non-empty structure, so trying to pop from empty stack will
    throw ValueError.
    """

    def push(self, el):
        return self.cons(el)

    def pop(self):
        if not self:
            raise ValueError("Stack is empty")
        return self.head, self.tail

    def is_empty(self):
        return not self


class Queue(object):
    """A queue is a particular kind of collection in which the entities in the
    collection are kept in order and the principal operations on the
    collection are the addition of entities to the rear terminal position,
    known as enqueue, and removal of entities from the front terminal position,
    known as dequeue.

    Queue data structure description on Wikipedia:
    [1] http://en.wikipedia.org/wiki/Queue_(abstract_data_type)

    Implementation based on two linked lists (left and right). Enqueue
    operation performs cons on right list (the end of the queue). Dequeue
    peeks first element from the left list (when possible), if left list is
    emptpy we populate left list with element from right one-by-one (in
    natural reverse order). Complexity of both operations are O(1).

    Such implementation is also known as "Banker's Queue" in different papers,
    i.e. in Chris Okasaki, "Purely Functional Data Structures"

    Usage:

    >>> from fn.immutable import Queue
    >>> q = Queue()
    >>> q1 = q.enqueue(10)
    >>> q2 = q1.enqueue(20)
    >>> el, tail = q2.dequeue()
    >>> el
    10
    >>> tail.dequeue()
    (20, <fn.immutable.list.Queue object at 0x1055554d0>)
    """

    __slots__ = ("left", "right")

    def __init__(self, left=None, right=None):
        self.left = left if left is not None else LinkedList()
        self.right = right if right is not None else LinkedList()

    def enqueue(self, el):
        """Returns new queue object with given element is added onto the end"""
        # check if we need to rebalance to prevent spikes
        if len(self.left) >= len(self.right):
            return Queue(self.left, self.right.cons(el))
        left = reduce(lambda acc, el: acc.cons(el), self.right, self.left)
        return Queue(left, LinkedList().cons(el))

    def dequeue(self):
        """Return pair of values: the item from the front of the queue and
        the new queue object without poped element.
        """
        if not self:
            raise ValueError("Queue is empty")
        # if there is at least one element on the left, we can return it
        if self.left:
            return self.left.head, Queue(self.left.tail, self.right)

        # in other case we need to copy right to left before
        d = reduce(lambda acc, el: acc.cons(el), self.right, LinkedList())
        return d.head, Queue(d.tail, LinkedList())

    def is_empty(self):
        return len(self) == 0

    def __iter__(self):
        curr = self
        while curr:
            el, curr = curr.dequeue()
            yield el

    def __nonzero__(self):
        return len(self)

    def __bool__(self):
        return len(self) > 0

    def __len__(self):
        lleft = len(self.left) if self.left is not None else 0
        lright = len(self.right) if self.right is not None else 0
        return lleft + lright


class Deque(object):
    """Double-ended queue is an  abstract data type that generalizes
    a queue, for which elements can be added to or removed from either
    the front (head) or back (tail).

    More information on Wikipedia:
    [1] http://en.wikipedia.org/wiki/Double-ended_queue

    Implementation details are described here:
    "Confluently Persistent Deques via Data Structural Bootstrapping"
    [2] https://cs.uwaterloo.ca/~imunro/cs840/p155-buchsbaum.pdf

    xxx: TBD
    """
    pass



"""Finger tree implementation and application examples.

A finger tree is a purely functional data structure used in
efficiently implementing other functional data structures. A
finger tree gives amortized constant time access to the "fingers"
(leaves) of the tree, where data is stored, and the internal
nodes are labeled in some way as to provide the functionality of
the particular data structure being implemented.

More information on Wikipedia: http://goo.gl/ppH2nE

"Finger trees: a simple general-purpose data structure": http://goo.gl/jX4DeL
"""

from collections import namedtuple

from fn.uniform import reduce

# data Node a = Node2 a a | Node3 a a a
# data Digit a = One a | Two a a | Three a a a | Four a a a a
# data FingerTree a = Empty
#                   | Single a
#                   | Deep (Digit a) (FingerTree (Node a)) (Digit a)

One = namedtuple("One", "a")
Two = namedtuple("Two", "a,b")
Three = namedtuple("Three", "a,b,c")
Four = namedtuple("Four", "a,b,c,d")


class Node2(namedtuple("Node2", "a,b")):
    def __iter__(self):
        yield self.a
        yield self.b


class Node3(namedtuple("Node3", "a,b,c")):
    def __iter__(self):
        yield self.a
        yield self.b
        yield self.c


class FingerTree(object):

    class Empty(object):
        __slots__ = ("measure",)

        def __init__(self, measure):
            object.__setattr__(self, "measure", measure)

        def __setattr__(self, *args):
            raise AttributeError("Attributes of {0} object "
                                 "cannot be changed".format("Empty"))

        def __delattr__(self, *args):
            raise AttributeError("Attributes of {0} object "
                                 "cannot be deleted".format("Empty"))

        def is_empty(self):
            return True

        def head(self):
            return None

        def last(self):
            return None

        def tail(self):
            return self

        def butlast(self):
            return self

        def push_front(self, v):
            return FingerTree.Single(self.measure, v)

        def push_back(self, v):
            return FingerTree.Single(self.measure, v)

        def __iter__(self):
            return iter([])

    class Single(object):
        __slots__ = ("measure", "elem",)

        def __init__(self, measure, elem):
            object.__setattr__(self, "measure", measure)
            object.__setattr__(self, "elem", elem)

        def __setattr__(self, *args):
            raise AttributeError("Attributes of {0} object "
                                 "cannot be changed".format("Single"))

        def __delattr__(self, *args):
            raise AttributeError("Attributes of {0} object "
                                 "cannot be deleted".format("Single"))

        def is_empty(self):
            return False

        def head(self):
            return self.elem

        def last(self):
            return self.elem

        def tail(self):
            return FingerTree.Empty(self.measure)

        def butlast(self):
            return FingerTree.Empty(self.measure)

        def push_front(self, v):
            return FingerTree.Deep(
                self.measure, [v], FingerTree.Empty(self.measure), [self.elem]
            )

        def push_back(self, v):
            return FingerTree.Deep(
                self.measure, [self.elem], FingerTree.Empty(self.measure), [v]
            )

        def __iter__(self):
            return iter([self.elem])

    class Deep(object):
        __slots__ = ("measure", "left", "middle", "right",)

        def __init__(self, measure, left, middle, right):
            object.__setattr__(self, "measure", measure)
            object.__setattr__(self, "left", left)
            object.__setattr__(self, "middle", middle)
            object.__setattr__(self, "right", right)

        def __setattr__(self, *args):
            raise AttributeError("Attributes of {0} object "
                                 "cannot be changed".format("Deep"))

        def __delattr__(self, *args):
            raise AttributeError("Attributes of {0} object "
                                 "cannot be deleted".format("Deep"))

        def is_empty(self):
            return False

        def head(self):
            return self.left[0]

        def last(self):
            return self.right[-1]

        def tail(self):
            if len(self.left) == 1:
                if self.middle.is_empty():
                    return FingerTree.from_iterable(
                        self.measure, list(self.right)
                    )
                return FingerTree.Deep(
                    self.measure,
                    [self.middle.head()],
                    self.middle.tail(),
                    self.right
                )
            return FingerTree.Deep(
                self.measure, self.left[1:], self.middle, self.right
            )

        def butlast(self):
            if len(self.rigth) == 1:
                if self.middle.is_empty():
                    return FingerTree.from_iterable(
                        self.measure, list(self.left)
                    )
                return FingerTree.Deep(
                    self.measure,
                    self.left,
                    self.middle.butlast(),
                    [self.middle.last()])
            return FingerTree.Deep(
                self.measure, self.left, self.middle, self.right[:-1]
            )

        def push_front(self, v):
            if len(self.left) == 4:
                return FingerTree.Deep(
                    self.measure,
                    [v, self.left[0]],
                    self.middle.push_front(Node3(*self.left[1:])),
                    self.right
                )
            return FingerTree.Deep(
                self.measure, [v] + self.left, self.middle, self.right
            )

        def push_back(self, v):
            if len(self.right) == 4:
                return FingerTree.Deep(
                    self.measure,
                    self.left,
                    self.middle.push_back(Node3(*self.right[:3])),
                    [self.right[-1], v])
            return FingerTree.Deep(
                self.measure, self.left, self.middle, self.right + [v]
            )

        def __iter__(self):
            for l in self.left:
                yield l
            for m in self.middle:
                for mi in m:
                    yield mi
            for r in self.right:
                yield r

    @staticmethod
    def from_iterable(measure, it):
        tree = FingerTree.Empty(measure)
        return reduce(lambda acc, curr: acc.push_front(curr), it, tree)

    def __new__(_cls, measure):
        return FingerTree.Empty(measure)

#####################################################
# Possible applications of finger tree in practice
#####################################################


class Deque(object):
    def __new__(_cls):
        return FingerTree.Empty(lambda x: x)

    @staticmethod
    def from_iterable(it):
        return FingerTree.from_iterable(lambda x: x, it)



from sys import version_info

# Syntax sugar to deal with Python 2/Python 3
# differences: this one will return generator
# even in Python 2.*
if version_info[0] == 2:
    from itertools import izip as zip, imap as map, ifilter as filter

filter = filter
map = map
zip = zip

if version_info[0] == 3:
    from functools import reduce

reduce = reduce
range = xrange if version_info[0] == 2 else range

if version_info[0] == 2:
    from itertools import ifilterfalse as filterfalse
    from itertools import izip_longest as zip_longest
else:
    from itertools import filterfalse
    from itertools import zip_longest

# Using or importing the ABCs from 'collections' instead of from
# 'collections.abc' is deprecated, and in 3.8 it will stop working.
if version_info[0] <= 3 and version_info[1] < 8:
    from collections import Iterable
else:
    from collections.abc import Iterable



#!/usr/bin/env python

import os
import sys

from pkg_resources import get_distribution, DistributionNotFound

try:
    get_distribution('fn')
    sys.stdout.write("""
{delimiter}
                   INSTALLATION ABORTED

The module "fn" was found on this system. Unfortunately
"fn.py" and "fn" cannot work together because they use the
same package name, i.e. fn. The "fn" module is no longer
maintained by the original author and "fn.py" is actually
a maintained fork of that project.

To complete this install, please uninstall "fn" (e.g.
pip uninstall fn) and rerun the installation of "fn.py".
{delimiter}
""".format(delimiter='=' * 60))
    sys.exit(0)
except DistributionNotFound:
    pass

import fn

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

short = '''
Implementation of missing features to enjoy functional programming in Python
'''
setup(
    name='fn.py',
    version=fn.__version__,
    description=short,
    long_description=(
        open('README.rst').read() +
        '\n\n' +
        open('CHANGELOG.rst').read()
    ),
    author='fnpy team',
    author_email='vash0the0stampede@gmail.com',
    url='https://github.com/fnpy/fn.py',
    packages=['fn', 'fn.immutable'],
    package_data={'': ['LICENSE', 'README.rst', 'CHANGELOG.rst']},
    include_package_data=True,
    install_requires=[],
    license=open('LICENSE').read(),
    zip_safe=False,
    keywords=['functional', 'fp', 'utility'],
    classifiers=(
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ),
)



from fn import Stream
from fn.iters import takewhile
from fn.uniform import reduce


class Vector(object):
    """A vector is a collection of values indexed by contiguous integers.
    Based on Philip Bagwell's "Array Mapped Trie" and Rick Hickey's
    "Bitmapped Vector Trie". As well as Clojure variant it supports access
    to items by index in log32N hops.

    Code structure inspired much by PersistenVector.java (Clojure core):
    [1] http://goo.gl/sqtZ74

    Usage:
    >>> from fn.immutable import Vector
    >>> v = Vector()
    >>> v1 = v.assoc(0, 10)
    >>> v2 = v1.assoc(1, 20)
    >>> v3 = v2.assoc(2, 30)
    >>> v3.get(0)
    10
    >>> v3.get(1)
    20
    >>> v3.get(2)
    30
    >>> v4 = v2.assoc(2, 50)
    >>> v3.get(2) # <-- previous version didn't change
    30
    >>> v4.get(2)
    50
    """

    __slots__ = ("length", "shift", "root", "tail")

    class _Node(object):
        __slots__ = ("array")

        def __init__(self, values=None, init=None):
            self.array = values or [None]*32
            if init is not None:
                self.array[0] = init

        def __str__(self):
            return str(self.array)

        def __iter__(self):
            s = reduce(
                lambda acc, el: acc << (
                    el if isinstance(el, self.__class__) else [el]
                ),
                takewhile(lambda el: el is not None, self.array),
                Stream()
            )
            return iter(s)

    def __init__(self, length=0, shift=None, root=None, tail=None):
        self.length = length
        self.shift = shift if shift is not None else 5
        self.root = root or self.__class__._Node()
        self.tail = tail if tail is not None else []

    def assoc(self, pos, el):
        """Returns a new vector that contains el at given position.
        Note, that position must be <= len(vector)
        """
        if pos < 0 or pos > self.length:
            raise IndexError()
        if pos == self.length:
            return self.cons(el)
        if pos < self._tailoff():
            up = self.__class__._do_assoc(self.shift, self.root, pos, el)
            return self.__class__(self.length, self.shift, up, self.tail)

        up = self.tail[:]
        up[pos & 0x01f] = el
        return self.__class__(self.length, self.shift, self.root, up)

    def _tailoff(self):
        if self.length < 32:
            return 0
        return ((self.length - 1) >> 5) << 5

    @classmethod
    def _do_assoc(cls, level, node, pos, el):
        r = cls._Node(node.array[:])
        if level == 0:
            r.array[pos & 0x01f] = el
        else:
            sub = (pos >> level) & 0x01f
            r.array[sub] = cls._do_assoc(level-5, node.array[sub], pos, el)
        return r

    def cons(self, el):
        # if there is a room in tail, just append value to tail
        if (self.length - self._tailoff()) < 32:
            tailup = self.tail[:]
            tailup.append(el)
            return self.__class__(self.length+1, self.shift, self.root, tailup)

        # if tail is already full, we need to push element into tree
        # (from the top)
        tailnode = self.__class__._Node(self.tail)

        # if root is overflowed, we need to expand the whole tree
        if (self.length >> 5) > (1 << self.shift):
            uproot = self.__class__._Node(init=self.root)
            uproot.array[1] = self.__class__._make_path(self.shift, tailnode)
            shift = self.shift + 5
        else:
            uproot = self._push_tail(self.shift, self.root, tailnode)
            shift = self.shift

        return self.__class__(self.length+1, shift, uproot, [el])

    def _push_tail(self, level, root, tail):
        sub = ((self.length - 1) >> level) & 0x01f
        r = self.__class__._Node(root.array[:])
        if level == 5:
            r.array[sub] = tail
        else:
            child = root.array[sub]
            if child is not None:
                r.array[sub] = self._push_tail(level-5, child, tail)
            else:
                r.array[sub] = self.__class__._make_path(level-5, tail)
        return r

    @classmethod
    def _make_path(cls, level, node):
        if level == 0:
            return node
        return cls._Node(init=cls._make_path(level-5, node))

    def get(self, pos):
        """Returns a value accossiated with position"""
        if pos < 0 or pos >= self.length:
            raise IndexError()
        return self._find_container(pos)[pos & 0x01f]

    def peek(self):
        """Returns the last item in vector or None if vector is empty"""
        if self.length == 0:
            return None
        return self.get(self.length-1)

    def pop(self):
        """Returns a new vector without the last item"""
        if self.length == 0:
            raise ValueError("Vector is empty")
        if self.length == 1:
            return self.__class__()
        if self.length - self._tailoff() > 1:
            return self.__class__(
                self.length-1, self.shift, self.root, self.tail[:-1]
            )

        tail = self._find_container(self.length - 2)
        root = self._pop_tail(self.shift, self.root) or self.__class__._Node()
        shift = self.shift

        if shift > 5 and root.array[1] is None:
            root = root.array[0]
            shift -= 5

        return self.__class__(self.length-1, shift, root, tail)

    def _find_container(self, pos):
        if pos < 0 or pos > self.length:
            raise IndexError()
        if pos >= self._tailoff():
            return self.tail
        bottom = reduce(lambda node, level: node.array[(pos >> level) & 0x01f],
                        range(self.shift, 0, -5), self.root)
        return bottom.array

    def _pop_tail(self, level, node):
        sub = ((self.length - 2) >> level) & 0x01f
        if level > 5:
            child = self._pop_tail(level-5, node.array[sub])
            if child is None and sub == 0:
                return None
            r = self.__class__._Node(node.array[:])
            r.array[sub] = child
            return r
        elif sub == 0:
            return None
        else:
            r = self.__class__._Node(node.array[:])
            r.array[sub] = None
            return r

    def subvec(self, start, end=None):
        """Returns a new vector of the items in vector from start to end"""
        pass

    def __len__(self):
        return self.length

    def __iter__(self):
        s = Stream() << self.root << self.tail
        return iter(s)

    def __getitem__(self, pos):
        return self.get(pos)

    def __setitem__(self, pos, val):
        raise NotImplementedError()



import itertools
import operator
import unittest

from fn import F, _, iters, op


class OperatorTestCase(unittest.TestCase):

    def test_currying(self):
        def add(first):
            def add(second):
                return first + second
            return add

        self.assertEqual(1, op.curry(add, 0, 1))

    def test_apply(self):
        self.assertEqual(10, op.apply(operator.add, [2, 8]))

    def test_flip(self):
        self.assertEqual(10, op.flip(operator.sub)(2, 12))
        self.assertEqual(-10, op.flip(op.flip(operator.sub))(2, 12))
        # flipping of flipped function should use optimization
        self.assertTrue(operator.sub is op.flip(op.flip(operator.sub)))

    def test_flip_with_shortcut(self):
        self.assertEqual(10, op.flip(_ - _)(2, 12))

    def test_zipwith(self):
        zipper = op.zipwith(operator.add)
        self.assertEqual(
            [10, 11, 12], list(zipper([0, 1, 2], itertools.repeat(10)))
        )

        zipper = op.zipwith(_ + _)
        self.assertEqual(
            [10, 11, 12], list(zipper([0, 1, 2], itertools.repeat(10)))
        )

        zipper = F() << list << op.zipwith(_ + _)
        self.assertEqual(
            [10, 11, 12], zipper([0, 1, 2], itertools.repeat(10))
        )

    def test_foldl(self):
        self.assertEqual(10, op.foldl(operator.add)([0, 1, 2, 3, 4]))
        self.assertEqual(20, op.foldl(operator.add, 10)([0, 1, 2, 3, 4]))
        self.assertEqual(20, op.foldl(operator.add, 10)(iters.range(5)))
        self.assertEqual(10, op.foldl(_ + _)(range(5)))

    def test_foldr(self):
        summer = op.foldr(operator.add)
        self.assertEqual(10, summer([0, 1, 2, 3, 4]))
        self.assertEqual(20, op.foldr(operator.add, 10)([0, 1, 2, 3, 4]))
        self.assertEqual(20, op.foldr(operator.add, 10)(iters.range(5)))
        # specific case for right-side folding
        self.assertEqual(
            100,
            op.foldr(op.call, 0)([lambda s: s**2, lambda k: k+10])
        )

    def test_unfold_infinite(self):
        doubler = op.unfold(lambda x: (x*2, x*2))
        self.assertEqual(20, next(doubler(10)))
        self.assertEqual(
            [20, 40, 80, 160, 320], list(iters.take(5, doubler(10)))
        )

    def test_unfold_finite(self):
        countdown = op.unfold(lambda x: (x-1, x-2) if x > 1 else None)
        self.assertEqual([9, 7, 5, 3, 1], list(countdown(10)))



import operator
import unittest

from fn import _, iters


class IteratorsTestCase(unittest.TestCase):

    def test_take(self):
        self.assertEqual([0, 1], list(iters.take(2, range(10))))
        self.assertEqual([0, 1], list(iters.take(10, range(2))))

    def test_drop(self):
        self.assertEqual([3, 4], list(iters.drop(3, range(5))))
        self.assertEqual([], list(iters.drop(10, range(2))))

    def test_first_true(self):
        pred = _ == 5
        self.assertEqual(5, iters.first_true(range(1, 10), pred=pred))
        self.assertEqual(
            999, iters.first_true(range(6, 10), default=999, pred=pred)
        )

    def test_takelast(self):
        self.assertEqual([8, 9], list(iters.takelast(2, range(10))))
        self.assertEqual([0, 1], list(iters.takelast(10, range(2))))

    def test_droplast(self):
        self.assertEqual([0, 1], list(iters.droplast(3, range(5))))
        self.assertEqual([], list(iters.droplast(10, range(2))))

    def test_consume(self):
        # full consuming, without limitation
        r = iters.range(10)
        self.assertEqual(10, len(list(r)))
        itr = iter(r)
        iters.consume(itr)
        self.assertEqual(0, len(list(itr)))

    def test_consume_limited(self):
        r = iters.range(10)
        self.assertEqual(10, len(list(r)))
        itr = iter(r)
        iters.consume(itr, 5)
        self.assertEqual(5, len(list(itr)))

    def test_nth(self):
        self.assertEqual(1, iters.nth(range(5), 1))
        self.assertEqual(None, iters.nth(range(5), 10))
        self.assertEqual("X", iters.nth(range(5), 10, "X"))

    def test_head(self):
        self.assertEqual(0, iters.head([0, 1, 2]))
        self.assertEqual(None, iters.head([]))

        def gen():
            yield 1
            yield 2
            yield 3

        self.assertEqual(1, iters.head(gen()))

    def test_first(self):
        self.assertEqual(iters.first, iters.head)  # Check if same object

    def test_tail(self):
        self.assertEqual([1, 2], list(iters.tail([0, 1, 2])))
        self.assertEqual([], list(iters.tail([])))

        def gen():
            yield 1
            yield 2
            yield 3

        self.assertEqual([2, 3], list(iters.tail(gen())))

    def test_rest(self):
        self.assertEqual(iters.rest, iters.tail)  # Check if same object

    def test_second(self):
        self.assertEqual(2, iters.second([1, 2, 3]))
        self.assertEqual(None, iters.second([]))

        def gen():
            yield 10
            yield 20
            yield 30

        self.assertEqual(20, iters.second(gen()))

    def test_ffirst(self):
        self.assertEqual(1, iters.ffirst([[1, 2], [3, 4]]))
        self.assertEqual(None, iters.ffirst([[], [10, 20]]))

        def gen():
            yield (x * 10 for x in (10, 20, 30,))

        self.assertEqual(100, iters.ffirst(gen()))

    def test_compact(self):
        self.assertEqual([True, 1, 0.1, "non-empty", [""], (0,), {"a": 1}],
                         list(iters.compact([None, False, True, 0, 1, 0.0, 0.1,
                                             "", "non-empty", [], [""],
                                             (), (0,), {}, {"a": 1}])))

    def test_every(self):
        self.assertEqual(True, iters.every(_ % 2 == 0, [2, 4, 6]))
        self.assertEqual(False, iters.every(_ % 2 == 0, [1, 3, 5]))
        self.assertEqual(False, iters.every(_ % 2 == 0, [2, 4, 6, 7]))

    def test_some(self):
        self.assertEqual("one",
                         iters.some(lambda k: {1: "one", 2: "two"}.get(k, ""),
                                    [1, 2]))
        self.assertEqual(None,
                         iters.some(lambda k: {1: "one", 2: "two"}.get(k, ""),
                                    [4, 3]))
        self.assertEqual("two",
                         iters.some(lambda k: {1: "one", 2: "two"}.get(k, ""),
                                    [4, 3, 2]))

    def test_reject(self):
        self.assertEqual(
            [1, 3, 5, 7, 9],
            list(iters.reject(_ % 2 == 0, range(1, 11)))
        )
        self.assertEqual(
            [None, False, 0, 0.0, "", [], (), {}],
            list(
                iters.reject(
                    None, [
                        None, False, True, 0, 1, 0.0, 0.1, "", "non-empty",
                        [], [""], (), (0,), {}, {"a": 1}
                    ])
            )
        )

    def test_iterate(self):
        it = iters.iterate(lambda x: x * x, 2)
        self.assertEqual(2, next(it))  # 2
        self.assertEqual(4, next(it))  # 2 * 2
        self.assertEqual(16, next(it))  # 4 * 4
        self.assertEqual(256, next(it))  # 16 * 16

    def test_padnone(self):
        it = iters.padnone([10, 11])
        self.assertEqual(10, next(it))
        self.assertEqual(11, next(it))
        self.assertEqual(None, next(it))
        self.assertEqual(None, next(it))

    def test_ncycles(self):
        it = iters.ncycles([10, 11], 2)
        self.assertEqual(10, next(it))
        self.assertEqual(11, next(it))
        self.assertEqual(10, next(it))
        self.assertEqual(11, next(it))
        self.assertRaises(StopIteration, next, it)

    def test_repeatfunc(self):
        def f():
            return "test"

        # unlimited count
        it = iters.repeatfunc(f)
        self.assertEqual("test", next(it))
        self.assertEqual("test", next(it))
        self.assertEqual("test", next(it))

        # limited
        it = iters.repeatfunc(f, 2)
        self.assertEqual("test", next(it))
        self.assertEqual("test", next(it))
        self.assertRaises(StopIteration, next, it)

    def test_grouper(self):
        # without fill value (default should be None)
        a, b, c = iters.grouper(3, "ABCDEFG")
        self.assertEqual(["A", "B", "C"], list(a))
        self.assertEqual(["D", "E", "F"], list(b))
        self.assertEqual(["G", None, None], list(c))

        # with fill value
        a, b, c = iters.grouper(3, "ABCDEFG", "x")
        self.assertEqual(["A", "B", "C"], list(a))
        self.assertEqual(["D", "E", "F"], list(b))
        self.assertEqual(["G", "x", "x"], list(c))

    def test_group_by(self):
        # verify grouping logic
        grouped = iters.group_by(len, ['1', '12', 'a', '123', 'ab'])
        self.assertEqual({1: ['1', 'a'], 2: ['12', 'ab'], 3: ['123']}, grouped)

        # verify it works with any iterable - not only lists
        def gen():
            yield '1'
            yield '12'

        grouped = iters.group_by(len, gen())
        self.assertEqual({1: ['1'], 2: ['12']}, grouped)

    def test_roundrobin(self):
        r = iters.roundrobin('ABC', 'D', 'EF')
        self.assertEqual(["A", "D", "E", "B", "F", "C"], list(r))

    def test_partition(self):
        def is_odd(x):
            return x % 2 == 1

        before, after = iters.partition(is_odd, iters.range(5))
        self.assertEqual([0, 2, 4], list(before))
        self.assertEqual([1, 3], list(after))

    def test_splitat(self):
        before, after = iters.splitat(2, iters.range(5))
        self.assertEqual([0, 1], list(before))
        self.assertEqual([2, 3, 4], list(after))

    def test_splitby(self):
        def is_even(x):
            return x % 2 == 0

        before, after = iters.splitby(is_even, iters.range(5))
        self.assertEqual([0], list(before))
        self.assertEqual([1, 2, 3, 4], list(after))

    def test_powerset(self):
        ps = iters.powerset([1, 2])
        self.assertEqual([tuple(), (1,), (2,), (1, 2)], list(ps))

    def test_pairwise(self):
        ps = iters.pairwise([1, 2, 3, 4])
        self.assertEqual([(1, 2), (2, 3), (3, 4)], list(ps))

    def test_iter_except(self):
        d = ["a", "b", "c"]
        it = iters.iter_except(d.pop, IndexError)
        self.assertEqual(["c", "b", "a"], list(it))

    def test_flatten(self):
        # flatten nested lists
        self.assertEqual([1, 2, 3, 4], list(iters.flatten([[1, 2], [3, 4]])))
        self.assertEqual(
            [1, 2, 3, 4, 5, 6],
            list(iters.flatten([[1, 2], [3, [4, 5, 6]]]))
        )
        # flatten nested tuples, sets, and frozen sets
        self.assertEqual(
            [1, 2, 3, 4, 5, 6],
            list(iters.flatten(((1, 2), (3, (4, 5, 6)))))
        )
        self.assertEqual(
            [1, 2, 3],
            list(iters.flatten(set([1, frozenset([2, 3])])))
        )
        # flatten nested generators
        generators = ((num + 1 for num in range(0, n)) for n in range(1, 4))
        self.assertEqual([1, 1, 2, 1, 2, 3], list(iters.flatten(generators)))
        # flat list should return itself
        self.assertEqual([1, 2, 3], list(iters.flatten([1, 2, 3])))
        # Don't flatten strings/unicode, bytes, or bytearrays
        self.assertEqual([2, "abc", 1], list(iters.flatten([2, "abc", 1])))
        self.assertEqual([2, b'abc', 1], list(iters.flatten([2, b'abc', 1])))
        self.assertEqual([2, bytearray(b'abc'), 1],
                         list(iters.flatten([2, bytearray(b'abc'), 1])))
        self.assertEqual(
            [bytearray(b'abc'), b'\xd1\x8f'.decode('utf8'), b'y'],
            list(iters.flatten(
                [bytearray(b'abc'), b'\xd1\x8f'.decode('utf8'), b'y']
            )))

    def test_accumulate(self):
        self.assertEqual(
            [1, 3, 6, 10, 15],
            list(iters.accumulate([1, 2, 3, 4, 5]))
        )
        self.assertEqual(
            [1, 2, 6, 24, 120],
            list(iters.accumulate([1, 2, 3, 4, 5], operator.mul))
        )

    def test_filterfalse(self):
        filtered = iters.filterfalse(lambda x: x > 10, [1, 2, 3, 11, 12])
        self.assertEqual([1, 2, 3], list(filtered))



import unittest

from fn.immutable import LinkedList, Stack


class LinkedListsTestCase(unittest.TestCase):

    def test_linked_list_basic_operations(self):
        l1 = LinkedList()
        l2 = l1.cons(1)
        l3 = l2.cons(2)
        self.assertEqual(None, l1.head)
        self.assertEqual(1, l2.head)
        self.assertEqual(2, l3.head)
        self.assertEqual(1, l3.tail.head)
        self.assertEqual(None, l3.tail.tail.head)

    def test_linked_list_num_of_elements(self):
        self.assertEqual(0, len(LinkedList()))
        self.assertEqual(3, len(LinkedList().cons(10).cons(20).cons(30)))

    def tests_linked_list_iterator(self):
        self.assertEqual(
            [30, 20, 10],
            list(LinkedList().cons(10).cons(20).cons(30))
        )

    def test_from_iterable(self):
        expected = [10, 20, 30]
        actual = list(LinkedList.from_iterable(expected))
        self.assertEqual(actual, expected)

        actual = LinkedList.from_iterable(tuple(expected))
        self.assertEqual(list(actual), expected)

        actual = LinkedList.from_iterable(iter(expected))
        self.assertEqual(list(actual), expected)

        actual = LinkedList.from_iterable(
            LinkedList().cons(30).cons(20).cons(10)
        )
        self.assertEqual(list(actual), expected)

    def test_stack_push_pop_ordering(self):
        s1 = Stack()
        s2 = s1.push(1)
        s3 = s2.push(10)
        s4 = s3.push(100)
        (sv4, s5) = s4.pop()
        (sv3, s6) = s5.pop()
        self.assertEqual(100, sv4)
        self.assertEqual(10, sv3)
        self.assertEqual(100, s4.pop()[0])

    def test_stack_length(self):
        self.assertEqual(0, len(Stack()))
        self.assertEqual(3, len(Stack().push(1).push(2).push(3)))

    def test_stack_is_empty_check(self):
        self.assertTrue(Stack().push(100))
        self.assertFalse(Stack().push(100).is_empty())
        self.assertTrue(Stack().is_empty())

    def test_pop_empty_stack_exception(self):
        self.assertRaises(ValueError, Stack().pop)

    def test_stack_iterator(self):
        self.assertEqual([10, 5, 1], list(Stack().push(1).push(5).push(10)))
        self.assertEqual(6, sum(Stack().push(1).push(2).push(3)))



from functools import partial

from fn.op import identity


default_cmp = (lambda a, b: -1 if (a < b) else 1)


class _MergeBased(object):

    def __nonzero__(self):
        return self.root is not None

    def __bool__(self):
        return self.__nonzero__()

    def __iter__(self):
        """Extract elements one-by-one.
        Note, that list(*Heap()) gives you sorted list as result.
        """
        curr = self
        while curr:
            r, curr = curr.extract()
            yield r

    def __lt__(self, other):
        if (not self) and (not other):
            return False
        if not self:
            return True
        if not other:
            return False
        return self.cmpfn(self.keyfn(self.root), self.keyfn(other.root)) < 0


class SkewHeap(_MergeBased):
    """A skew heap (or self-adjusting heap) is a heap data structure
    implemented as a binary-tree. Amortized complexity analytics can
    be used to demonstrate that all operations one a skew heap can be
    done in O(log n).

    Skew heaps may be described with the following recursive definition:
     * a heap with only one element is a skew heap
     * the result of skew merging two skew heaps is also a skew heap

    In Haskell type definition it should looks like following::

        data Skew a = Empty | Node a (Skew a) (Skew a)

    More information on Wikipedia:
    [1] http://en.wikipedia.org/wiki/Skew_heap

    One can also check slides from my KyivPy#11 talk "Union-based heaps":
    [2] http://goo.gl/VMgdG2

    Basic usage sample:

    >>> from fn.immutable import SkewHeap
    >>> s = SkewHeap(10)
    >>> s = s.insert(20)
    >>> s
    <fn.immutable.heap.SkewHeap object at 0x10b14c050>
    >>> s = s.insert(30)
    >>> s
    <fn.immutable.heap.SkewHeap object at 0x10b14c158> # <-- other object
    >>> s.extract()
    (10, <fn.immutable.heap.SkewHeap object at 0x10b14c050>)
    >>> _, s = s.extract()
    >>> s.extract()
    (20, <fn.immutable.heap.SkewHeap object at 0x10b14c1b0>)
    """

    __slots__ = ("root", "left", "right", "keyfn", "cmpfn", "_make_heap")

    def __init__(self, el=None, left=None, right=None, key=None, cmp=None):
        """Creates skew heap with one element (or empty one)"""
        self.root = el
        self.left = left
        self.right = right
        self.keyfn = key or identity
        self.cmpfn = cmp or default_cmp
        self._make_heap = partial(
            self.__class__, key=self.keyfn, cmp=self.cmpfn
        )

    def insert(self, el):
        """Returns new skew heap with additional element"""
        return self._make_heap(el).union(self)

    def extract(self):
        """Returns pair of values:
        * minimum (or maximum regarding to given compare function)
        * new skew heap without extracted element

        Or ``None`` and empty heap if ``self`` is an empty heap.
        """
        if not self:
            return None, self._make_heap()
        if self.left:
            tmp_heap = self.left.union(self.right)
        else:
            tmp_heap = self._make_heap()
        return self.root, tmp_heap

    def union(self, other):
        """Merge two heaps and returns new one (skew merging)"""
        if not self:
            return other
        if not other:
            return self

        if self < other:
            return self._make_heap(
                self.root, other.union(self.right), self.left
            )
        return self._make_heap(other.root, self.union(other.right), other.left)


class PairingHeap(_MergeBased):
    """A pairing heap is either an empty heap, or a pair consisting of a root
    element and a possibly empty list of pairing heap. The heap ordering
    property requires that all the root elements of the subheaps in the list
    are not smaller (bigger) than the root element of the heap.

    In Haskell type definition it should looks like following::

        data Pairing a = Empty | Node a [Pairing a]

    Pairing heap has and excellent practical amortized performance. The
    amortized time per extract is less than O(log n), find-min/find-max, merge
    and insert are O(1).

    More information about performance bounds you can find here:
    "The Pairing Heap: A New Form of Self-Adjusting Heap"
    [1]
http://www.cs.cmu.edu/afs/cs.cmu.edu/user/sleator/www/papers/pairing-heaps.pdf

    More general information on Wikipedia:
    [2] http://en.wikipedia.org/wiki/Pairing_heap

    One can also check slides from my KyivPy#11 talk "Union-based heaps":
    [3] http://goo.gl/VMgdG2

    Basic usage sample:

    >>> from fn.immutable import PairingHeap
    >>> ph = PairingHeap("a")
    >>> ph = ph.insert("b")
    >>> ph
    <fn.immutable.heap.PairingHeap object at 0x10b13fa00>
    >>> ph = ph.insert("c")
    >>> ph
    <fn.immutable.heap.PairingHeap object at 0x10b13fa50>
    >>> ph.extract()
    ('a', <fn.immutable.heap.PairingHeap object at 0x10b13fa00>)
    >>> _, ph = ph.extract()
    >>> ph.extract()
    ('b', <fn.immutable.heap.PairingHeap object at 0x10b13f9b0>)
    """

    __slots__ = ("root", "subs", "keyfn", "cmpfn", "_make_heap")

    def __init__(self, el=None, subs=None, key=None, cmp=None):
        """Creates singlton from given element
        (pairing heap with one element or empty one)
        """
        self.root = el
        self.subs = subs
        self.keyfn = key or identity
        self.cmpfn = cmp or default_cmp
        self._make_heap = partial(
            self.__class__, key=self.keyfn, cmp=self.cmpfn
        )

    def insert(self, el):
        """Returns new pairing heap with additional element"""
        return self.union(self._make_heap(el))

    def extract(self):
        """Returns pair of values:
         * minimum (or maximum regarding to given compare function)
         * new pairing heap without extracted element

        Or None and empty heap if self is an empty heap.
        """
        if not self:
            return None, self._make_heap()
        return self.root, PairingHeap._pairing(self._make_heap, self.subs)

    def union(self, other):
        """Returns new heap as a result of merging two given

        Note, that originally this operation for pairingi heap was
        named "meld", see [1] and [2]. We use here name "union" to
        follow consistent naming convention for all heap implementations.
        """
        if not self:
            return other
        if not other:
            return self

        if self < other:
            return self._make_heap(self.root, (other, self.subs))
        return self._make_heap(other.root, (self, other.subs))

    @staticmethod
    def _pairing(heap, hs):
        if hs is None:
            return heap()
        (h1, tail) = hs
        if tail is None:
            return h1
        (h2, tail) = tail
        return PairingHeap._pairing(heap, (h1.union(h2), tail))



import operator
import unittest

from fn import F, _, iters


class CompositionTestCase(unittest.TestCase):

    def test_composition(self):
        def f(x):
            return x * 2

        def g(x):
            return x + 10

        self.assertEqual(30, (F(f) << g)(5))

        def z(x):
            return x * 20
        self.assertEqual(220, (F(f) << F(g) << F(z))(5))

    def test_partial(self):
        # Partial should work if we pass additional arguments to F constructor
        f = F(operator.add, 10) << F(operator.add, 5)
        self.assertEqual(25, f(10))

    def test_underscore(self):
        self.assertEqual(
            [1, 4, 9],
            list(map(F() << (_ ** 2) << _ + 1, range(3)))
        )

    def test_pipe_composition(self):
        def f(x):
            return x * 2

        def g(x):
            return x + 10

        self.assertEqual(20, (F() >> f >> g)(5))

    def test_pipe_partial(self):
        func = F() >> (iters.filter, _ < 6) >> sum
        self.assertEqual(15, func(iters.range(10)))
